@@ -1,6 +1,6 @@
 
 /* GConf
- * Copyright (C) 1999 Red Hat Inc.
+ * Copyright (C) 1999, 2000 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
 #include "gconf-value.h"
 #include "gconf-error.h"
 #include "gconf-schema.h"
+#include "gconf-internals.h"
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
@@ -53,10 +54,12 @@ set_string(gchar** dest, const gchar* src)
  */
 
 GConfValue* 
-g_conf_value_new(GConfValueType type)
+gconf_value_new(GConfValueType type)
 {
   GConfValue* value;
 
+  g_return_val_if_fail(GCONF_INTERNAL_VALUE_TYPE_VALID(type), NULL);
+  
   /* Probably want to use mem chunks here eventually. */
   value = g_new0(GConfValue, 1);
 
@@ -69,16 +72,16 @@ g_conf_value_new(GConfValueType type)
 }
 
 GConfValue* 
-g_conf_value_new_from_string(GConfValueType type, const gchar* value_str,
+gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
                              GConfError** err)
 {
   GConfValue* value;
 
-  value = g_conf_value_new(type);
+  value = gconf_value_new(type);
 
   switch (type)
     {
-    case G_CONF_VALUE_INT:
+    case GCONF_VALUE_INT:
       {
         char* endptr = NULL;
         glong result;
@@ -89,51 +92,50 @@ g_conf_value_new_from_string(GConfValueType type, const gchar* value_str,
         if (endptr == value_str)
           {
             if (err)
-              *err = g_conf_error_new(G_CONF_PARSE_ERROR,
+              *err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
                                       _("Didn't understand `%s' (expected integer)"),
                                       value_str);
             
-            g_conf_value_destroy(value);
+            gconf_value_destroy(value);
             value = NULL;
           }
         else if (errno == ERANGE)
           {
             if (err)
-              *err = g_conf_error_new(G_CONF_PARSE_ERROR,
+              *err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
                                       _("Integer `%s' is too large or small"),
                                       value_str);
-            g_conf_value_destroy(value);
+            gconf_value_destroy(value);
             value = NULL;
           }
         else
-          g_conf_value_set_int(value, result);
+          gconf_value_set_int(value, result);
       }
       break;
-    case G_CONF_VALUE_FLOAT:
+    case GCONF_VALUE_FLOAT:
       {
-        gchar* endptr = 0;
         double num;
-        num = g_strtod(value_str, &endptr);
-        if (value_str != endptr)
+
+        if (gconf_string_to_double(value_str, &num))
           {
-            g_conf_value_set_float(value, num);
+            gconf_value_set_float(value, num);
           }
         else
           {
             if (err)
-              *err = g_conf_error_new(G_CONF_PARSE_ERROR,
+              *err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
                                       _("Didn't understand `%s' (expected real number)"),
-                                      value_str);
+                                     value_str);
             
-            g_conf_value_destroy(value);
+            gconf_value_destroy(value);
             value = NULL;
           }
       }
       break;
-    case G_CONF_VALUE_STRING:
-      g_conf_value_set_string(value, value_str);
+    case GCONF_VALUE_STRING:
+      gconf_value_set_string(value, value_str);
       break;
-    case G_CONF_VALUE_BOOL:
+    case GCONF_VALUE_BOOL:
       switch (*value_str)
         {
         case 't':
@@ -141,7 +143,7 @@ g_conf_value_new_from_string(GConfValueType type, const gchar* value_str,
         case '1':
         case 'y':
         case 'Y':
-          g_conf_value_set_bool(value, TRUE);
+          gconf_value_set_bool(value, TRUE);
           break;
 
         case 'f':
@@ -149,23 +151,22 @@ g_conf_value_new_from_string(GConfValueType type, const gchar* value_str,
         case '0':
         case 'n':
         case 'N':
-          g_conf_value_set_bool(value, FALSE);
+          gconf_value_set_bool(value, FALSE);
           break;
           
         default:
           if (err)
-            *err = g_conf_error_new(G_CONF_PARSE_ERROR,
-                                    _("Didn't understand `%s' (expected true or false)"),
-                                    value_str);
+            *err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+                                   _("Didn't understand `%s' (expected true or false)"),
+                                   value_str);
           
-          g_conf_value_destroy(value);
+          gconf_value_destroy(value);
           value = NULL;
           break;
         }
       break;
-    case G_CONF_VALUE_LIST:
-    case G_CONF_VALUE_PAIR:
-    case G_CONF_VALUE_IGNORE_SUBSEQUENT:
+    case GCONF_VALUE_LIST:
+    case GCONF_VALUE_PAIR:
     default:
       g_assert_not_reached();
       break;
@@ -175,13 +176,13 @@ g_conf_value_new_from_string(GConfValueType type, const gchar* value_str,
 }
 
 GConfValue*
-g_conf_value_new_list_from_string(GConfValueType list_type,
+gconf_value_new_list_from_string(GConfValueType list_type,
                                   const gchar* str)
 {
-  g_return_val_if_fail(list_type != G_CONF_VALUE_LIST, NULL);
-  g_return_val_if_fail(list_type != G_CONF_VALUE_PAIR, NULL);
+  g_return_val_if_fail(list_type != GCONF_VALUE_LIST, NULL);
+  g_return_val_if_fail(list_type != GCONF_VALUE_PAIR, NULL);
 
-  /* FIXME (parse the string generated by g_conf_value_to_string();
+  /* FIXME (parse the string generated by gconf_value_to_string();
      probably need to add string escaping to that function to make
      lists/pairs of string work */
   
@@ -189,14 +190,14 @@ g_conf_value_new_list_from_string(GConfValueType list_type,
 }
 
 GConfValue*
-g_conf_value_new_pair_from_string(GConfValueType car_type,
+gconf_value_new_pair_from_string(GConfValueType car_type,
                                   GConfValueType cdr_type,
                                   const gchar* str)
 {
-  g_return_val_if_fail(car_type != G_CONF_VALUE_LIST, NULL);
-  g_return_val_if_fail(car_type != G_CONF_VALUE_PAIR, NULL);
+  g_return_val_if_fail(car_type != GCONF_VALUE_LIST, NULL);
+  g_return_val_if_fail(car_type != GCONF_VALUE_PAIR, NULL);
 
-  /* FIXME (parse the string generated by g_conf_value_to_string();
+  /* FIXME (parse the string generated by gconf_value_to_string();
      probably need to add string escaping to that function to make
      lists/pairs of string work */
 
@@ -204,7 +205,7 @@ g_conf_value_new_pair_from_string(GConfValueType car_type,
 }
 
 gchar*
-g_conf_value_to_string(GConfValue* value)
+gconf_value_to_string(GConfValue* value)
 {
   /* These strings shouldn't be translated; they're primarily 
      intended for machines to read, not humans, though I do
@@ -214,70 +215,77 @@ g_conf_value_to_string(GConfValue* value)
 
   switch (value->type)
     {
-    case G_CONF_VALUE_INT:
-      retval = g_malloc(64);
-      g_snprintf(retval, 64, "%d", g_conf_value_int(value));
+    case GCONF_VALUE_INT:
+      retval = g_strdup_printf("%d", gconf_value_int(value));
       break;
-    case G_CONF_VALUE_FLOAT:
-      retval = g_malloc(64);
-      g_snprintf(retval, 64, "%g", g_conf_value_float(value));
+    case GCONF_VALUE_FLOAT:
+      retval = gconf_double_to_string(gconf_value_float(value));
       break;
-    case G_CONF_VALUE_STRING:
-      retval = g_strdup(g_conf_value_string(value));
+    case GCONF_VALUE_STRING:
+      retval = g_strdup(gconf_value_string(value));
       break;
-    case G_CONF_VALUE_BOOL:
-      retval = g_conf_value_bool(value) ? g_strdup("true") : g_strdup("false");
+    case GCONF_VALUE_BOOL:
+      retval = gconf_value_bool(value) ? g_strdup("true") : g_strdup("false");
       break;
-    case G_CONF_VALUE_LIST:
+    case GCONF_VALUE_LIST:
       {
         GSList* list;
 
-        list = g_conf_value_list(value);
+        list = gconf_value_list(value);
 
         if (list == NULL)
           retval = g_strdup("[]");
         else
           {
-            gchar* buf;
-            guint bufsize = 128;
+            gchar* buf = NULL;
+            guint bufsize = 64;
             guint cur = 0;
 
             g_assert(list != NULL);
             
-            buf = g_malloc(bufsize);
+            buf = g_malloc(bufsize+3); /* my +3 superstition */
             
             buf[0] = '[';
             ++cur;
 
+            g_assert(cur < bufsize);
+            
             while (list != NULL)
               {
                 gchar* elem;
                 guint len;
                 
-                elem = g_conf_value_to_string((GConfValue*)list->data);
+                elem = gconf_value_to_string((GConfValue*)list->data);
 
                 g_assert(elem != NULL);
 
                 len = strlen(elem);
 
-                if ((cur + len) >= bufsize)
+                if ((cur + len + 2) >= bufsize) /* +2 for '\0' and comma */
                   {
-                    bufsize *= 2;
-                    buf = g_realloc(buf, bufsize);
-                    buf[bufsize-1] = '\0'; /* paranoia */
+                    bufsize = MAX(bufsize*2, bufsize+len+4); 
+                    buf = g_realloc(buf, bufsize+3);
                   }
+
+                g_assert(cur < bufsize);
                 
                 strcpy(&buf[cur], elem);
                 cur += len;
 
+                g_assert(cur < bufsize);
+                
                 g_free(elem);
 
                 buf[cur] = ',';
                 ++cur;
+
+                g_assert(cur < bufsize);
                 
                 list = g_slist_next(list);
               }
 
+            g_assert(cur < bufsize);
+            
             buf[cur-1] = ']'; /* overwrites last comma */
             buf[cur] = '\0';
 
@@ -285,27 +293,33 @@ g_conf_value_to_string(GConfValue* value)
           }
       }
       break;
-    case G_CONF_VALUE_PAIR:
+    case GCONF_VALUE_PAIR:
       {
         gchar* car;
         gchar* cdr;
 
-        car = g_conf_value_to_string(g_conf_value_car(value));
-        cdr = g_conf_value_to_string(g_conf_value_cdr(value));
+        car = gconf_value_to_string(gconf_value_car(value));
+        cdr = gconf_value_to_string(gconf_value_cdr(value));
         retval = g_strdup_printf("(%s, %s)", car, cdr);
         g_free(car);
         g_free(cdr);
       }
       break;
       /* These remaining shouldn't really be used outside of debug spew... */
-    case G_CONF_VALUE_INVALID:
+    case GCONF_VALUE_INVALID:
       retval = g_strdup("Invalid");
       break;
-    case G_CONF_VALUE_SCHEMA:
-      retval = g_strdup("Schema");
-      break;
-    case G_CONF_VALUE_IGNORE_SUBSEQUENT:
-      retval = g_strdup("Ignore Subsequent");
+    case GCONF_VALUE_SCHEMA:
+      {
+        const gchar* locale;
+        const gchar* type;
+        
+        locale = gconf_schema_locale(gconf_value_schema(value));
+        type = gconf_value_type_to_string(gconf_schema_type(gconf_value_schema(value)));
+        
+        retval = g_strdup_printf("Schema (type: `%s' locale: `%s')",
+                                 type, locale ? locale : "(null)");
+      }
       break;
     default:
       g_assert_not_reached();
@@ -323,7 +337,7 @@ copy_value_list(GSList* list)
   
   while (tmp != NULL)
     {
-      copy = g_slist_prepend(copy, g_conf_value_copy(tmp->data));
+      copy = g_slist_prepend(copy, gconf_value_copy(tmp->data));
       
       tmp = g_slist_next(tmp);
     }
@@ -334,33 +348,33 @@ copy_value_list(GSList* list)
 }
 
 GConfValue* 
-g_conf_value_copy(GConfValue* src)
+gconf_value_copy(GConfValue* src)
 {
   GConfValue* dest;
 
   g_return_val_if_fail(src != NULL, NULL);
 
-  dest = g_conf_value_new(src->type);
+  dest = gconf_value_new(src->type);
 
   switch (src->type)
     {
-    case G_CONF_VALUE_INT:
-    case G_CONF_VALUE_FLOAT:
-    case G_CONF_VALUE_BOOL:
-    case G_CONF_VALUE_INVALID:
+    case GCONF_VALUE_INT:
+    case GCONF_VALUE_FLOAT:
+    case GCONF_VALUE_BOOL:
+    case GCONF_VALUE_INVALID:
       dest->d = src->d;
       break;
-    case G_CONF_VALUE_STRING:
+    case GCONF_VALUE_STRING:
       set_string(&dest->d.string_data, src->d.string_data);
       break;
-    case G_CONF_VALUE_SCHEMA:
+    case GCONF_VALUE_SCHEMA:
       if (src->d.schema_data)
-        dest->d.schema_data = g_conf_schema_copy(src->d.schema_data);
+        dest->d.schema_data = gconf_schema_copy(src->d.schema_data);
       else
         dest->d.schema_data = NULL;
       break;
       
-    case G_CONF_VALUE_LIST:
+    case GCONF_VALUE_LIST:
       {
         GSList* copy;
 
@@ -371,22 +385,19 @@ g_conf_value_copy(GConfValue* src)
       }
       break;
       
-    case G_CONF_VALUE_PAIR:
+    case GCONF_VALUE_PAIR:
 
       if (src->d.pair_data.car)
-        dest->d.pair_data.car = g_conf_value_copy(src->d.pair_data.car);
+        dest->d.pair_data.car = gconf_value_copy(src->d.pair_data.car);
       else
         dest->d.pair_data.car = NULL;
 
       if (src->d.pair_data.cdr)
-        dest->d.pair_data.cdr = g_conf_value_copy(src->d.pair_data.cdr);
+        dest->d.pair_data.cdr = gconf_value_copy(src->d.pair_data.cdr);
       else
         dest->d.pair_data.cdr = NULL;
 
       break;
-
-    case G_CONF_VALUE_IGNORE_SUBSEQUENT:
-      break;
       
     default:
       g_assert_not_reached();
@@ -396,17 +407,17 @@ g_conf_value_copy(GConfValue* src)
 }
 
 static void
-g_conf_value_free_list(GConfValue* value)
+gconf_value_free_list(GConfValue* value)
 {
   GSList* tmp;
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_LIST);
+  g_return_if_fail(value->type == GCONF_VALUE_LIST);
   
   tmp = value->d.list_data.list;
 
   while (tmp != NULL)
     {
-      g_conf_value_destroy(tmp->data);
+      gconf_value_destroy(tmp->data);
       
       tmp = g_slist_next(tmp);
     }
@@ -416,29 +427,29 @@ g_conf_value_free_list(GConfValue* value)
 }
 
 void 
-g_conf_value_destroy(GConfValue* value)
+gconf_value_destroy(GConfValue* value)
 {
   g_return_if_fail(value != NULL);
 
   switch (value->type)
     {
-    case G_CONF_VALUE_STRING:
+    case GCONF_VALUE_STRING:
       if (value->d.string_data != NULL)
         g_free(value->d.string_data);
       break;
-    case G_CONF_VALUE_SCHEMA:
+    case GCONF_VALUE_SCHEMA:
       if (value->d.schema_data != NULL)
-        g_conf_schema_destroy(value->d.schema_data);
+        gconf_schema_destroy(value->d.schema_data);
       break;
-    case G_CONF_VALUE_LIST:
-      g_conf_value_free_list(value);
+    case GCONF_VALUE_LIST:
+      gconf_value_free_list(value);
       break;
-    case G_CONF_VALUE_PAIR:
+    case GCONF_VALUE_PAIR:
       if (value->d.pair_data.car != NULL)
-        g_conf_value_destroy(value->d.pair_data.car);
+        gconf_value_destroy(value->d.pair_data.car);
 
       if (value->d.pair_data.cdr != NULL)
-        g_conf_value_destroy(value->d.pair_data.cdr);
+        gconf_value_destroy(value->d.pair_data.cdr);
       break;
     default:
       break;
@@ -448,116 +459,116 @@ g_conf_value_destroy(GConfValue* value)
 }
 
 void        
-g_conf_value_set_int(GConfValue* value, gint the_int)
+gconf_value_set_int(GConfValue* value, gint the_int)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_INT);
+  g_return_if_fail(value->type == GCONF_VALUE_INT);
 
   value->d.int_data = the_int;
 }
 
 void        
-g_conf_value_set_string(GConfValue* value, const gchar* the_str)
+gconf_value_set_string(GConfValue* value, const gchar* the_str)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_STRING);
+  g_return_if_fail(value->type == GCONF_VALUE_STRING);
 
   set_string(&value->d.string_data, the_str);
 }
 
 void        
-g_conf_value_set_float(GConfValue* value, gdouble the_float)
+gconf_value_set_float(GConfValue* value, gdouble the_float)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_FLOAT);
+  g_return_if_fail(value->type == GCONF_VALUE_FLOAT);
 
   value->d.float_data = the_float;
 }
 
 void        
-g_conf_value_set_bool(GConfValue* value, gboolean the_bool)
+gconf_value_set_bool(GConfValue* value, gboolean the_bool)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_BOOL);
+  g_return_if_fail(value->type == GCONF_VALUE_BOOL);
 
   value->d.bool_data = the_bool;
 }
 
 void       
-g_conf_value_set_schema(GConfValue* value, GConfSchema* sc)
+gconf_value_set_schema(GConfValue* value, GConfSchema* sc)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_SCHEMA);
+  g_return_if_fail(value->type == GCONF_VALUE_SCHEMA);
   
   if (value->d.schema_data != NULL)
-    g_conf_schema_destroy(value->d.schema_data);
+    gconf_schema_destroy(value->d.schema_data);
 
-  value->d.schema_data = g_conf_schema_copy(sc);
+  value->d.schema_data = gconf_schema_copy(sc);
 }
 
 void        
-g_conf_value_set_schema_nocopy(GConfValue* value, GConfSchema* sc)
+gconf_value_set_schema_nocopy(GConfValue* value, GConfSchema* sc)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_SCHEMA);
+  g_return_if_fail(value->type == GCONF_VALUE_SCHEMA);
   g_return_if_fail(sc != NULL);
   
   if (value->d.schema_data != NULL)
-    g_conf_schema_destroy(value->d.schema_data);
+    gconf_schema_destroy(value->d.schema_data);
 
   value->d.schema_data = sc;
 }
 
 void
-g_conf_value_set_car(GConfValue* value, GConfValue* car)
+gconf_value_set_car(GConfValue* value, GConfValue* car)
 {
   g_return_if_fail(car != NULL);
 
-  g_conf_value_set_car_nocopy(value, g_conf_value_copy(car));
+  gconf_value_set_car_nocopy(value, gconf_value_copy(car));
 }
 
 void
-g_conf_value_set_car_nocopy(GConfValue* value, GConfValue* car)
+gconf_value_set_car_nocopy(GConfValue* value, GConfValue* car)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_PAIR);
+  g_return_if_fail(value->type == GCONF_VALUE_PAIR);
   g_return_if_fail(car != NULL);
   
   if (value->d.pair_data.car != NULL)
-    g_conf_value_destroy(value->d.pair_data.car);
+    gconf_value_destroy(value->d.pair_data.car);
 
   value->d.pair_data.car = car;
 }
 
 void
-g_conf_value_set_cdr(GConfValue* value, GConfValue* cdr)
+gconf_value_set_cdr(GConfValue* value, GConfValue* cdr)
 {
   g_return_if_fail(cdr != NULL);
 
-  g_conf_value_set_cdr_nocopy(value, g_conf_value_copy(cdr));
+  gconf_value_set_cdr_nocopy(value, gconf_value_copy(cdr));
 }
 
 void
-g_conf_value_set_cdr_nocopy(GConfValue* value, GConfValue* cdr)
+gconf_value_set_cdr_nocopy(GConfValue* value, GConfValue* cdr)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_PAIR);
+  g_return_if_fail(value->type == GCONF_VALUE_PAIR);
   g_return_if_fail(cdr != NULL);
   
   if (value->d.pair_data.cdr != NULL)
-    g_conf_value_destroy(value->d.pair_data.cdr);
+    gconf_value_destroy(value->d.pair_data.cdr);
 
   value->d.pair_data.cdr = cdr;
 }
 
 void
-g_conf_value_set_list_type(GConfValue* value,
+gconf_value_set_list_type(GConfValue* value,
                            GConfValueType type)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_LIST);
-  g_return_if_fail(type != G_CONF_VALUE_LIST);
-  g_return_if_fail(type != G_CONF_VALUE_PAIR);
+  g_return_if_fail(value->type == GCONF_VALUE_LIST);
+  g_return_if_fail(type != GCONF_VALUE_LIST);
+  g_return_if_fail(type != GCONF_VALUE_PAIR);
   /* If the list is non-NULL either we already have the right
      type, or we shouldn't be changing it without deleting
      the list first. */
@@ -567,32 +578,32 @@ g_conf_value_set_list_type(GConfValue* value,
 }
 
 void
-g_conf_value_set_list_nocopy(GConfValue* value,
+gconf_value_set_list_nocopy(GConfValue* value,
                              GSList* list)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_LIST);
-  g_return_if_fail(value->d.list_data.type != G_CONF_VALUE_INVALID);
+  g_return_if_fail(value->type == GCONF_VALUE_LIST);
+  g_return_if_fail(value->d.list_data.type != GCONF_VALUE_INVALID);
   
   if (value->d.list_data.list)
-    g_conf_value_free_list(value);
+    gconf_value_free_list(value);
 
   value->d.list_data.list = list;
 }
 
 void
-g_conf_value_set_list       (GConfValue* value,
+gconf_value_set_list       (GConfValue* value,
                              GSList* list)
 {
   g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == G_CONF_VALUE_LIST);
-  g_return_if_fail(value->d.list_data.type != G_CONF_VALUE_INVALID);
+  g_return_if_fail(value->type == GCONF_VALUE_LIST);
+  g_return_if_fail(value->d.list_data.type != GCONF_VALUE_INVALID);
   g_return_if_fail((list == NULL) ||
                    ((list->data != NULL) &&
                     (((GConfValue*)list->data)->type == value->d.list_data.type)));
   
   if (value->d.list_data.list)
-    g_conf_value_free_list(value);
+    gconf_value_free_list(value);
 
   value->d.list_data.list = copy_value_list(list);
 }
@@ -602,7 +613,7 @@ g_conf_value_set_list       (GConfValue* value,
  */
 
 GConfMetaInfo*
-g_conf_meta_info_new(void)
+gconf_meta_info_new(void)
 {
   GConfMetaInfo* gcmi;
 
@@ -614,27 +625,27 @@ g_conf_meta_info_new(void)
 }
 
 void
-g_conf_meta_info_destroy(GConfMetaInfo* gcmi)
+gconf_meta_info_destroy(GConfMetaInfo* gcmi)
 {
   g_free(gcmi);
 }
 
 void
-g_conf_meta_info_set_schema  (GConfMetaInfo* gcmi,
+gconf_meta_info_set_schema  (GConfMetaInfo* gcmi,
                               const gchar* schema_name)
 {
   set_string(&gcmi->schema, schema_name);
 }
 
 void
-g_conf_meta_info_set_mod_user(GConfMetaInfo* gcmi,
+gconf_meta_info_set_mod_user(GConfMetaInfo* gcmi,
                               const gchar* mod_user)
 {
   set_string(&gcmi->mod_user, mod_user);
 }
 
 void
-g_conf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
+gconf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
                               GTime mod_time)
 {
   gcmi->mod_time = mod_time;
@@ -645,7 +656,7 @@ g_conf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
  */
 
 GConfEntry* 
-g_conf_entry_new(gchar* key, GConfValue* val)
+gconf_entry_new_nocopy(gchar* key, GConfValue* val)
 {
   GConfEntry* pair;
 
@@ -653,14 +664,54 @@ g_conf_entry_new(gchar* key, GConfValue* val)
 
   pair->key   = key;
   pair->value = val;
-
+  pair->schema_name = NULL;
+  pair->is_default = FALSE;
+  
   return pair;
 }
 
 void
-g_conf_entry_destroy(GConfEntry* pair)
+gconf_entry_destroy(GConfEntry* pair)
 {
   g_free(pair->key);
-  g_conf_value_destroy(pair->value);
+  if (pair->value)
+    gconf_value_destroy(pair->value);
   g_free(pair);
 }
+
+GConfValue*
+gconf_entry_steal_value (GConfEntry* entry)
+{
+  GConfValue* val = entry->value;
+  entry->value = NULL;
+  return val;
+}
+
+void
+gconf_entry_set_value_nocopy(GConfEntry* entry,
+                             GConfValue* val)
+{
+  if (entry->value)
+    gconf_value_destroy(entry->value);
+
+  entry->value = val;
+}
+
+void
+gconf_entry_set_schema_name(GConfEntry* entry,
+                            const gchar* name)
+{
+  if (entry->schema_name)
+    g_free(entry->schema_name);
+
+  entry->schema_name = name ? g_strdup(name) : NULL;
+}
+
+void
+gconf_entry_set_is_default (GConfEntry* entry,
+                            gboolean is_default)
+{
+  entry->is_default = is_default;
+}
+
+
